@@ -1,14 +1,15 @@
 """ Hook regular expression patterns up to method calls on a scanner object. """
-from boozetools.support.interfaces import Scanner
-
+from ..support.interfaces import Scanner
+from ..arborist import trees
 from ..support import interfaces
 from ..parsing import miniparse
 from . import finite, regular, charset, recognition
 
-PRELOAD = {'ASCII': {k: regular.CharSpecial(cls) for k, cls in charset.mode_ascii.items()}}
+char_prebuilt = regular.VOCAB['CharPrebuilt']
+PRELOAD = {'ASCII': {k: char_prebuilt.leaf(cls) for k, cls in charset.mode_ascii.items()}}
 
 
-class Definition(interfaces.ScanRules):
+class Definition:
 	def __init__(self, *, minimize=True, mode='ASCII'):
 		self.__actions = []
 		self.__trails = []
@@ -29,15 +30,15 @@ class Definition(interfaces.ScanRules):
 	
 	def scan(self, text, *, start=None, on_error:interfaces.ScanErrorListener = None):
 		if on_error is None: on_error = interfaces.ScanErrorListener()
-		scanner = recognition.IterableScanner(text=text, automaton=self.get_dfa(), rules=self, start=start, on_error=on_error)
+		scanner = recognition.IterableScanner(text=text, automaton=self.get_dfa(), act=self.invoke, start=start, on_error=on_error)
 		return scanner
 		
-	def install_subexpression(self, name:str, expression: regular.Regular):
+	def install_subexpression(self, name:str, expression: trees.Node):
 		assert isinstance(name, str) and name not in self.__subexpressions and len(name) > 1
-		assert isinstance(expression, regular.Regular)
+		assert isinstance(expression, trees.Node), type(expression)
 		self.__subexpressions[name] = expression
 	
-	def install_rule(self, *, action:callable, expression: regular.Regular, bol=(True, True), condition: (str, list, tuple)=None, trail:int=None, rank:int=0) -> int:
+	def install_rule(self, *, action:callable, expression: trees.Node, bol=(True, True), condition: (str, list, tuple)=None, trail:int=None, rank:int=0) -> int:
 		rule_id = len(self.__actions)
 		self.__actions.append(action)
 		self.__trails.append(trail)
@@ -49,7 +50,7 @@ class Definition(interfaces.ScanRules):
 			for q, b in zip(self.__nfa.condition(C), bol):
 				if b: self.__nfa.link_epsilon(q,src)
 		self.__nfa.final[dst] = rule_id
-		regular.Encoder(self.__nfa, rank, self.__subexpressions).visit(expression, src, dst)
+		expression.tour(regular.Encoder(self.__nfa, rank, self.__subexpressions), src, dst)
 		return rule_id
 	
 	def let(self, name:str, pattern:str):
@@ -90,26 +91,26 @@ class Definition(interfaces.ScanRules):
 	def condition(self, *condition): return ConditionContext(self, *condition)
 	
 	def invoke(self, yy: interfaces.Scanner, rule_id:int):
+		trail = self.get_trailing_context(rule_id)
+		if trail is not None: yy.less(trail)
+
 		action = self.__actions[rule_id]
 		if callable(action): action(yy)
 		else: assert action is None
 	
-
-
-def analyze_pattern(pattern:str, env):
-	scanner = META.scan(pattern)
-	bol, expression, trailing_context = rex.parse(scanner)
-	assert isinstance(expression, regular.Regular)
-	if not trailing_context: trail = None
-	else:
-		assert isinstance(trailing_context, regular.Regular), trailing_context
+def analyze_pattern(pattern_text, env):
+	pattern = rex.parse(META.scan(pattern_text), language='Pattern')
+	if pattern.symbol.label == 'pattern_regular': expression, trail = pattern['stem'], None
+	elif pattern.symbol.label == 'pattern_only_trail': expression, trail = pattern['trail'], 0
+	elif pattern.symbol.label == 'pattern_with_trail':
 		sizer = regular.Sizer(env)
-		stem, trail = sizer.visit(expression), sizer.visit(trailing_context)
-		expression = regular.Sequence(expression, trailing_context)
+		stem, trail = pattern['stem'].tour(sizer), pattern['trail'].tour(sizer)
+		expression = regular.VOCAB['Sequence'].from_args(pattern['stem'], pattern['trail'])
 		if trail: trail = -trail
 		elif stem: trail = stem
 		else: raise regular.TrailingContextError('Variable stem and variable trailing context in the same pattern are not presently supported.')
-	return bol, expression, trail
+	else: assert False, pattern.symbol.label
+	return regular.LEFT_CONTEXT[pattern['left_context'].symbol.label], expression, trail
 
 class ConditionContext:
 	""" I'd like to be able to use Python's context manager protocol to simplify writing definitions of scan conditions. """
@@ -126,52 +127,61 @@ class ConditionContext:
 
 #########################
 # A pattern parser is easy to build using the miniparse module:
+class RegexParser(miniparse.MiniParse):
+	def did_not_recover(self):
+		raise regular.PatternSyntaxError
 
-rex = miniparse.MiniParse('Pattern', 'Regular')
-rex.void_symbols.update('^ ^^ / | ? * + { } , ( ) [ - ] &&'.split())
-rex.rule('Pattern', 'BOL Regular Trail')()
-rex.rule('Pattern', 'BOL $')(lambda bol, eof:(bol, eof, None)) # To specify end-of-file rules without introducing yet another metacharacter
-rex.rule('BOL', '')(lambda : (True, True))
-rex.rule('BOL', '^')(lambda : (False, True))
-rex.rule('BOL', '^^')(lambda : (True, False))
-rex.rule('Trail', '')()
-rex.rule('Trail', '$')()
-rex.rule('Trail', '/ Regular')()
-rex.rule('Trail', '/ Regular $')(regular.Sequence)
-rex.rule('Regular', 'Sequence')()
-rex.rule('Regular', 'Regular | Sequence')(regular.Alternation)
-rex.rule('Sequence', 'Term')()
-rex.rule('Sequence', 'Sequence Term')(regular.Sequence)
-rex.rule('Term', 'Atom')()
-rex.rule('Term', 'Atom ?')(regular.Hook)
-rex.rule('Term', 'Atom *')(regular.Star)
-rex.rule('Term', 'Atom +')(regular.Plus)
-rex.rule('Term', 'Atom { number }')(lambda a, n: regular.Counted(a, n, n))
-rex.rule('Term', 'Atom { number , }')(lambda a, n: regular.Counted(a, n, None))
-rex.rule('Term', 'Atom { , number }')(lambda a, n: regular.Counted(a, 0, n))
-rex.rule('Term', 'Atom { number , number }')(lambda a, m, n: regular.Counted(a, m, n))
-rex.rule('Atom', '( Regular )')()
-rex.rule('Atom', 'c')(regular.Letter)
-rex.rule('Atom', 'reference')()
-rex.rule('Atom', '[ Class ]')()
-rex.rule('Class', 'Conjunct')()
-rex.rule('Class', 'Class && Conjunct')(regular.CharIntersection)
-rex.rule('Conjunct', 'Members')()
-rex.rule('Conjunct', '^ Members')(regular.CharComplement)
-rex.rule('Members', 'Item')()
-rex.rule('Members', 'Members Item')(regular.CharUnion)
-rex.rule('Item', 'c')(regular.Letter)
-rex.rule('Item', 'c - c')(regular.CharRange)
-rex.rule('Item', 'short')()
-rex.rule('Item', 'reference')()
+	def __init__(self):
+		super(RegexParser, self).__init__('Pattern', 'Regular')
+		self.void_symbols.update('^ ^^ / | ? * + { } , ( ) [ - ] && ; whitespace'.split())
+		for line in """
+			Pattern: left_context Regular               :pattern_regular
+			Pattern: left_context Regular right_context :pattern_with_trail
+			Pattern: left_context right_context         :pattern_only_trail
+			left_context:    :anywhere
+			left_context: ^  :begin_line
+			left_context: ^^ :mid_line
+			right_context: end
+			right_context: / Regular
+			right_context: / Regular end :Sequence
+			Regular: sequence
+			Regular: Regular | sequence :Alternation
+			sequence: term
+			sequence: sequence term :Sequence
+			term: atom
+			term: atom ? :Hook
+			term: atom * :Star
+			term: atom + :Plus
+			term: atom { number }          :n_times
+			term: atom { number , }        :n_or_more
+			term: atom { , number }        :n_or_fewer
+			term: atom { number , number } :n_to_m
+			atom: codepoint
+			atom: reference
+			atom: ( Regular )
+			atom: [ class ]
+			class: conjunct
+			class: class && conjunct :CharIntersection
+			conjunct: members
+			conjunct: ^ members :CharComplement
+			members: item
+			members: members item :CharUnion
+			item: codepoint
+			item: codepoint - codepoint :CharRange
+			item: reference
+		""".splitlines():
+			bits = [x.strip() for x in line.split(':')]
+			if len(bits)==2: self.rule(*bits)()
+			elif len(bits)==3:
+				symbol = regular.VOCAB[bits.pop()]
+				self.rule(*bits)(symbol.from_args)
 
-META = Definition()
-def _BEGIN_():
+def _BOOTSTRAP_REGEX_SCANNER_():
 	""" This is the bootstrapping routine: it builds the preload and the meta-scanner. """
 	def seq(head, *tail):
-		for t in tail: head = regular.Sequence(head, t)
+		for t in tail: head = regular.VOCAB['Sequence'].from_args(head, t)
 		return head
-	def txt(s):return seq(*(regular.CharSpecial(charset.singleton(ord(_))) for _ in s))
+	def txt(s):return seq(*(regular.VOCAB['Codepoint'].leaf(ord(_)) for _ in s))
 	
 	def _metatoken(yy): yy.token(yy.matched_text(), None)
 	def _and_then(condition):
@@ -186,26 +196,26 @@ def _BEGIN_():
 		return fn
 	def _bracket_reference(yy:interfaces.Scanner):
 		name = yy.matched_text()[1:-1]
-		node = regular.NamedSubexpression(name, yy.current_span())
+		node = regular.VOCAB['NamedSubexpression'].leaf(name, yy.current_span())
 		yy.token('reference', node)
 	def _shorthand_reference(yy:interfaces.Scanner):
-		yy.token('reference', regular.NamedSubexpression(yy.matched_text()[1], yy.current_span()))
+		yy.token('reference', regular.VOCAB['NamedSubexpression'].leaf(yy.matched_text()[1], yy.current_span()))
 	def _dot_reference(yy:interfaces.Scanner):
-		yy.token('reference', regular.NamedSubexpression('DOT', yy.current_span()))
-	def _hex_escape(yy): yy.token('c', int(yy.matched_text()[2:], 16))
-	def _control(yy): yy.token('c', 31 & ord(yy.matched_text()[2:]))
-	def _arbitrary_character(yy): yy.token('c', ord(yy.matched_text()))
+		yy.token('reference', regular.VOCAB['NamedSubexpression'].leaf('DOT', yy.current_span()))
+	def _hex_escape(yy): yy.token('codepoint', regular.VOCAB['Codepoint'].leaf(int(yy.matched_text()[2:], 16)))
+	def _control(yy): yy.token('codepoint', regular.VOCAB['Codepoint'].leaf(31 & ord(yy.matched_text()[2:])))
+	def _arbitrary_character(yy): yy.token('codepoint', regular.VOCAB['Codepoint'].leaf(ord(yy.matched_text())))
 	def _class_initial_close_bracket(yy):
 		yy.enter('in_class')
 		_arbitrary_character(yy)
 	def _class_final_dash(yy):
-		yy.token('c', ord('-'))
+		yy.token('codepoint', ord('-'))
 		yy.token(']', None)
 		yy.enter(None)
-	def _arbitrary_escape(yy): yy.token('c', ord(yy.matched_text()[1:]))
-	def _number(yy): yy.token('number', int(yy.matched_text()))
+	def _arbitrary_escape(yy): yy.token('codepoint', regular.VOCAB['Codepoint'].leaf(ord(yy.matched_text()[1:])))
+	def _number(yy): yy.token('number', regular.VOCAB['Bound'].leaf(int(yy.matched_text())))
 	def _dollar(charclass):
-		def fn(yy:Scanner): yy.token('$', charclass)
+		def fn(yy:Scanner): yy.token('end', charclass)
 		return fn
 	def _meta_caret(yy):
 		yy.token(yy.matched_text()[:-1], None)
@@ -219,41 +229,47 @@ def _BEGIN_():
 	
 	dot = ref('DOT')
 	
-	eof_charclass = regular.CharSpecial(charset.EOF)
-	dollar_charclass = regular.CharSpecial(charset.union(charset.EOF, PRELOAD['ASCII']['vertical'].cls))
-	
-	for t in '^', '^^': META.install_rule(expression=txt(t), action=_metatoken, bol=(False, True))
+	eof_charclass = regular.VOCAB['CharPrebuilt'].leaf(charset.EOF)
+	dollar_charclass = regular.VOCAB['CharPrebuilt'].leaf(charset.union(charset.EOF, PRELOAD['ASCII']['vertical'].semantic))
+
+	meta = Definition()
+
+	for t in '^', '^^': meta.install_rule(expression=txt(t), action=_metatoken, bol=(False, True))
 	for t,cc in ('$', dollar_charclass), ('<<EOF>>', eof_charclass):
-		META.install_rule(expression=seq(txt(t), eof_charclass), trail=-1, action=_dollar(cc))
-	common_rules(META,
+		meta.install_rule(expression=seq(txt(t), eof_charclass), trail=-1, action=_dollar(cc))
+	common_rules(meta,
 		('.', _dot_reference),
 		('{', _and_then('brace')),
 		('[', _and_then('start_class')),
 		('[^', _meta_caret),
 		*((c, _metatoken) for c in '(|)?*+/'),
 	)
-	common_rules(META.condition('in_class'),
+	common_rules(meta.condition('in_class'),
 		(']', _and_then(None)),
 		('&&', _metatoken),
 		('&&^', _meta_caret),
 		('-', _metatoken),
 		('-]', _class_final_dash),
 	)
-	with META.condition('start_class') as start_class:
+	with meta.condition('start_class') as start_class:
 		common_rules(start_class, (']', _class_initial_close_bracket), ('-', _arbitrary_character),)
 		start_class.install_rule(expression=ref('ANY'), action = _instead('in_class'))
-	with META.condition(None, 'in_class') as anywhere:
-		anywhere.install_rule(expression=seq(txt('{'), ref('alpha'), regular.Plus(ref('word')), txt('}'), ), action=_bracket_reference)
-		whack = txt('\\')
+	with meta.condition(None, 'in_class') as anywhere:
+		anywhere.install_rule(expression=seq(txt('{'), ref('alpha'), regular.VOCAB['Plus'].from_args(ref('word')), txt('}'), ), action=_bracket_reference)
+		whack = txt('\\') # NB: Python doesn't let you end a raw-string with a backslash.
 		for c, n in [('x', 2), ('u', 4), ('U', 8)]:
-			META.install_rule(expression=seq(whack, txt(c), regular.Counted(ref('xdigit'), n, n)), action=_hex_escape)
-		anywhere.install_rule(expression=seq(whack, txt('c'), regular.CharSpecial(charset.range_class(64, 127))), action=_control)
+			hexblock = regular.VOCAB['n_times'].from_args(ref('xdigit'), regular.VOCAB['Bound'].leaf(n))
+			anywhere.install_rule(expression=seq(whack, txt(c), hexblock), action=_hex_escape)
+		anywhere.install_rule(expression=seq(whack, txt('codepoint'), regular.VOCAB['CharPrebuilt'].leaf(charset.range_class(64, 127))), action=_control)
 		anywhere.install_rule(expression=seq(whack, ref('alnum')), action=_shorthand_reference)
 		anywhere.install_rule(expression=seq(whack, dot), action=_arbitrary_escape)
 		anywhere.install_rule(expression=dot, action=_arbitrary_character)
-	with META.condition('brace') as brace:
-		brace.install_rule(expression=regular.Plus(ref('digit')), action=_number)
+	with meta.condition('brace') as brace:
+		brace.install_rule(expression=regular.VOCAB['Plus'].from_args(ref('digit')), action=_number)
 		common_rules(brace, (',', _metatoken), ('}', _and_then(None)),)
-	
-	PRELOAD['ASCII']['R'] = rex.parse(META.scan(r'\r?\n|\r'), language='Regular')
-_BEGIN_()
+	return meta
+
+rex = RegexParser()
+META = _BOOTSTRAP_REGEX_SCANNER_()
+PRELOAD['ASCII']['R'] = rex.parse(META.scan(r'\r?\n|\r'), language='Regular')
+
